@@ -3,18 +3,15 @@
 # Programmer friendly subprocess wrapper.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: May 30, 2015
+# Last Change: October 2, 2015
 # URL: https://executor.readthedocs.org
 
 """
-The :mod:`executor` module
-==========================
+Core functionality of the `executor` package.
 
-The :mod:`executor` module defines the core functionality of the ``executor``
-package. If you're looking for an easy way to run external commands from Python
-take a look at the :func:`execute()` function. When you need more flexibility
-consider using the underlying :class:`ExternalCommand` class directly
-instead.
+If you're looking for an easy way to run external commands from Python take a
+look at the :func:`execute()` function. When you need more flexibility consider
+using the underlying :class:`ExternalCommand` class directly instead.
 
 :func:`execute()` versus :class:`ExternalCommand`
 -------------------------------------------------
@@ -60,7 +57,7 @@ from executor.property_manager import (
 )
 
 # Semi-standard module versioning.
-__version__ = '4.4'
+__version__ = '4.5'
 
 # Initialize a logger.
 logger = logging.getLogger(__name__)
@@ -111,14 +108,14 @@ def execute(*command, **options):
                     :class:`ExternalCommand`.
     :returns: The return value of this function depends on two options:
 
-              =======================================  =========================================  =================================
-              Value of :attr:`~ExternalCommand.async`  Value of :attr:`~ExternalCommand.capture`  Return value
-              =======================================  =========================================  =================================
-              :data:`False`                            :data:`False`                              :attr:`ExternalCommand.succeeded`
-              :data:`False`                            :data:`True`                               :attr:`ExternalCommand.output`
-              :data:`True`                             :data:`True`                               :class:`ExternalCommand` object
-              :data:`True`                             :data:`False`                              :class:`ExternalCommand` object
-              =======================================  =========================================  =================================
+              ==============================  ================================  =================================
+              :attr:`~ExternalCommand.async`  :attr:`~ExternalCommand.capture`  Return value
+              ==============================  ================================  =================================
+              :data:`False`                   :data:`False`                     :attr:`ExternalCommand.succeeded`
+              :data:`False`                   :data:`True`                      :attr:`ExternalCommand.output`
+              :data:`True`                    :data:`True`                      :class:`ExternalCommand` object
+              :data:`True`                    :data:`False`                     :class:`ExternalCommand` object
+              ==============================  ================================  =================================
     :raises: :exc:`ExternalCommandFailed` when the command exits with a
              nonzero exit code (unless :attr:`~ExternalCommand.capture` is
              :data:`False`).
@@ -188,18 +185,22 @@ class ExternalCommand(PropertyManager):
     methods here is a summary:
 
     **Writable properties**
-     The :attr:`async`, :attr:`capture`, :attr:`check`, :attr:`command`,
+     The :attr:`async`, :attr:`capture`, :attr:`capture_stderr`, :attr:`check`,
      :attr:`directory`, :attr:`encoding`, :attr:`environment`,
-     :attr:`fakeroot`, :attr:`input`, :attr:`logger`, :attr:`silent` and
-     :attr:`sudo` properties allow you to configure how the external command
-     will be run (before it is started).
+     :attr:`fakeroot`, :attr:`input`, :attr:`logger`, :attr:`merge_streams`,
+     :attr:`silent`, :attr:`stdout_file`, :attr:`stderr_file` and :attr:`sudo`
+     properties allow you to configure how the external command will be run
+     (before it is started).
 
     **Computed properties**
-     The :attr:`command_line`, :attr:`encoded_input`, :attr:`is_finished`,
-     :attr:`is_running`, :attr:`is_terminated`, :attr:`output`,
-     :attr:`returncode`, :attr:`stdout` and :attr:`was_started` properties
-     allow you to inspect if and how the external command was started, what its
-     current status is and what its output is.
+     The :attr:`command`, :attr:`command_line`, :attr:`decoded_stderr`,
+     :attr:`decoded_stdout`, :attr:`encoded_input`, :attr:`error_message`,
+     :attr:`error_type`, :attr:`failed`, :attr:`have_superuser_privileges`,
+     :attr:`is_finished`, :attr:`is_running`, :attr:`is_terminated`,
+     :attr:`output`, :attr:`returncode`, :attr:`stderr`, :attr:`stdout`,
+     :attr:`succeeded` and :attr:`was_started` properties allow you to inspect
+     if and how the external command was started, what its current status is
+     and what its output is.
 
     **Public methods**
      The public methods :func:`start()`, :func:`wait()` and :func:`terminate()`
@@ -261,6 +262,8 @@ class ExternalCommand(PropertyManager):
     @mutable_property
     def async(self):
         """
+        Enable asynchronous command execution.
+
         If this option is :data:`True` (not the default) preparations are made
         to execute the external command asynchronously (in the background).
         This has several consequences:
@@ -291,6 +294,8 @@ class ExternalCommand(PropertyManager):
     @mutable_property
     def capture(self):
         """
+        Enable capturing of the standard output stream.
+
         If this option is :data:`True` (not the default) the standard output of
         the external command is captured and made available to the caller via
         :attr:`stdout` and :attr:`output`.
@@ -304,6 +309,8 @@ class ExternalCommand(PropertyManager):
     @mutable_property
     def capture_stderr(self):
         """
+        Enable capturing of the standard error stream.
+
         If this option is :data:`True` (not the default) the standard error
         stream of the external command is captured and made available to the
         caller via :attr:`stderr`.
@@ -313,6 +320,8 @@ class ExternalCommand(PropertyManager):
     @mutable_property
     def check(self):
         """
+        Enable automatic status code checking.
+
         If this option is :data:`True` (the default) and the external command
         exits with a nonzero status code :exc:`ExternalCommandFailed` will be
         raised by :func:`start()` (when :attr:`async` isn't set) or
@@ -327,6 +336,8 @@ class ExternalCommand(PropertyManager):
     @property
     def command_line(self):
         """
+        The command line of the external command.
+
         The command line used to actually run the external command requested by
         the user (a list of strings). The command line is constructed based on
         :attr:`command` according to the following rules:
@@ -358,9 +369,10 @@ class ExternalCommand(PropertyManager):
     @property
     def decoded_stdout(self):
         """
-        The value of :attr:`stdout` decoded using :attr:`encoding`. This is a
-        :func:`python2:unicode` object (in Python 2) or a :class:`python3:str`
-        object (in Python 3).
+        The value of :attr:`stdout` decoded using :attr:`encoding`.
+
+        This is a :func:`python2:unicode` object (in Python 2) or a
+        :class:`python3:str` object (in Python 3).
         """
         value = self.stdout
         if value is not None:
@@ -369,9 +381,10 @@ class ExternalCommand(PropertyManager):
     @property
     def decoded_stderr(self):
         """
-        The value of :attr:`stderr` decoded using :attr:`encoding`. This is a
-        :func:`python2:unicode` object (in Python 2) or a :class:`python3:str`
-        object (in Python 3).
+        The value of :attr:`stderr` decoded using :attr:`encoding`.
+
+        This is a :func:`python2:unicode` object (in Python 2) or a
+        :class:`python3:str` object (in Python 3).
         """
         value = self.stderr
         if value is not None:
@@ -380,17 +393,19 @@ class ExternalCommand(PropertyManager):
     @mutable_property
     def directory(self):
         """
-        The working directory for the external command (a string, defaults to
-        :data:`DEFAULT_WORKING_DIRECTORY`).
+        The working directory for the external command.
+
+        A string, defaults to :data:`DEFAULT_WORKING_DIRECTORY`.
         """
         return DEFAULT_WORKING_DIRECTORY
 
     @property
     def encoded_input(self):
         """
-        The value of :attr:`input` encoded using :attr:`encoding`. This is a
-        :class:`python2:str` object (in Python 2) or a :class:`python3:bytes`
-        object (in Python 3).
+        The value of :attr:`input` encoded using :attr:`encoding`.
+
+        This is a :class:`python2:str` object (in Python 2) or a
+        :class:`python3:bytes` object (in Python 3).
         """
         return (self.input.encode(self.encoding)
                 if isinstance(self.input, str)
@@ -399,9 +414,10 @@ class ExternalCommand(PropertyManager):
     @mutable_property
     def encoding(self):
         """
-        The character encoding of standard input and standard output (a string,
-        defaults to :data:`DEFAULT_ENCODING`). This option is used to encode
-        :attr:`input` and to decode :attr:`output`.
+        The character encoding of standard input and standard output.
+
+        A string, defaults to :data:`DEFAULT_ENCODING`. This option is used to
+        encode :attr:`input` and to decode :attr:`output`.
         """
         return DEFAULT_ENCODING
 
@@ -418,25 +434,22 @@ class ExternalCommand(PropertyManager):
 
     @property
     def error_message(self):
-        """
-        A string describing how the external command failed or :data:`None`.
-        """
+        """A string describing how the external command failed or :data:`None`."""
         if self.error_type is ExternalCommandFailed:
             text = "External command failed with exit code %s! (command: %s)"
             return text % (self.returncode, quote(self.command_line))
 
     @property
     def error_type(self):
-        """
-        :exc:`ExternalCommandFailed` when :attr:`returncode` is set and not
-        zero, :data:`None` otherwise.
-        """
+        """:exc:`ExternalCommandFailed` when :attr:`returncode` is a nonzero number, :data:`None` otherwise."""
         if self.is_finished and self.failed:
             return ExternalCommandFailed
 
     @property
     def failed(self):
         """
+        Whether the external command has failed.
+
         :data:`True` if :attr:`returncode` is a nonzero number, :data:`False`
         if :attr:`returncode` is zero, :data:`None` when the external command
         hasn't been started or is still running.
@@ -446,6 +459,8 @@ class ExternalCommand(PropertyManager):
     @mutable_property
     def fakeroot(self):
         """
+        Run the external command under ``fakeroot``.
+
         If this option is :data:`True` (not the default) and the current
         process doesn't have `superuser privileges`_ the external command is
         run with ``fakeroot``. If the ``fakeroot`` program is not installed a
@@ -458,6 +473,8 @@ class ExternalCommand(PropertyManager):
     @property
     def have_superuser_privileges(self):
         """
+        Whether the parent Python process is running under `superuser privileges`_.
+
         :data:`True` if running with `superuser privileges`_, :data:`False`
         otherwise. Used by :attr:`command_line` to decide whether
         :attr:`fakeroot` or :attr:`sudo` needs to be used.
@@ -467,14 +484,13 @@ class ExternalCommand(PropertyManager):
     @mutable_property
     def input(self):
         """
-        The input to feed to the external command on the standard input stream
-        (defaults to :data:`None`).
+        The input to feed to the external command on the standard input stream.
 
-        When you provide a :func:`python2:unicode` object (in Python 2) or a
-        :class:`python3:str` object (in Python 3) as input it will be encoded
-        using :attr:`encoding`. To avoid the automatic conversion you can
-        simply pass a :class:`python2:str` object (in Python 2) or a
-        :class:`python3:bytes` object (in Python 3).
+        Defaults to :data:`None`. When you provide a :func:`python2:unicode`
+        object (in Python 2) or a :class:`python3:str` object (in Python 3) as
+        input it will be encoded using :attr:`encoding`. To avoid the automatic
+        conversion you can simply pass a :class:`python2:str` object (in Python
+        2) or a :class:`python3:bytes` object (in Python 3).
 
         The conversion logic is implemented in the :attr:`encoded_input`
         attribute.
@@ -483,6 +499,8 @@ class ExternalCommand(PropertyManager):
     @property
     def is_finished(self):
         """
+        Whether the external command has finished execution.
+
         :data:`True` once the external command has been started and has since
         finished, :data:`False` when the external command hasn't been started
         yet or is still running.
@@ -492,6 +510,8 @@ class ExternalCommand(PropertyManager):
     @property
     def is_running(self):
         """
+        Whether the external command is currently running.
+
         :data:`True` while the external command is running, :data:`False` when
         the external command hasn't been started yet or has already finished.
         """
@@ -500,6 +520,8 @@ class ExternalCommand(PropertyManager):
     @property
     def is_terminated(self):
         """
+        Whether the external command has been terminated.
+
         :data:`True` if the external command was terminated using
         :data:`signal.SIGTERM` (e.g. by :func:`terminate()`),
         :data:`False` otherwise.
@@ -522,18 +544,21 @@ class ExternalCommand(PropertyManager):
     @mutable_property
     def merge_streams(self):
         """
-        Whether to merge the standard output and error streams (a boolean,
-        defaults to :data:`False`). If this option is enabled :attr:`stdout`
-        will contain the external command's output on both streams.
+        Whether to merge the standard output and error streams.
+
+        A boolean, defaults to :data:`False`. If this option is enabled
+        :attr:`stdout` will contain the external command's output on both
+        streams.
         """
         return False
 
     @property
     def output(self):
-        r"""
-        The value of :attr:`stdout` decoded using :attr:`encoding`. This is a
-        :func:`python2:unicode` object (in Python 2) or a :class:`python3:str`
-        object (in Python 3).
+        """
+        The value of :attr:`stdout` decoded using :attr:`encoding`.
+
+        This is a :func:`python2:unicode` object (in Python 2) or a
+        :class:`python3:str` object (in Python 3).
 
         This is only available when :attr:`capture` is :data:`True`. If
         :attr:`capture` is not :data:`True` then :attr:`output` will be
@@ -545,12 +570,12 @@ class ExternalCommand(PropertyManager):
         otherwise the decoded string is returned unchanged:
 
         >>> from executor import ExternalCommand
-        >>> cmd = ExternalCommand('echo naïve', capture=True)
+        >>> cmd = ExternalCommand('echo na\xc3\xafve', capture=True)
         >>> cmd.start()
         >>> cmd.output
-        u'na\xefve'
+        u'na\\xefve'
         >>> cmd.stdout
-        'na\xc3\xafve\n'
+        'na\\xc3\\xafve\\n'
 
         This is intended to make simple things easy (:attr:`output` makes it
         easy to deal with external commands that output a single line) while
@@ -565,14 +590,18 @@ class ExternalCommand(PropertyManager):
     @property
     def returncode(self):
         """
-        The return code of the external command (an integer). When the external
-        command hasn't finished yet :data:`None` is returned.
+        The return code of the external command (an integer).
+
+        When the external command hasn't finished yet :data:`None` is
+        returned.
         """
         return self.subprocess.poll() if self.subprocess else None
 
     @mutable_property
     def silent(self):
         """
+        Whether the external command's output should be silenced.
+
         If this is :data:`True` (not the default) any output of the external
         command is silenced by redirecting the output streams to
         :data:`os.devnull`.
@@ -585,9 +614,10 @@ class ExternalCommand(PropertyManager):
     @property
     def stderr(self):
         """
-        The output of the external command on its standard error stream, a
-        :class:`python2:str` object (in Python 2) or a :class:`python3:bytes`
-        object (in Python 3).
+        The output of the external command on its standard error stream.
+
+        This is a :class:`python2:str` object (in Python 2) or a
+        :class:`python3:bytes` object (in Python 3).
 
         This is only available when :attr:`capture_stderr` is :data:`True`. If
         :attr:`capture_stderr` is not :data:`True` then :attr:`stderr` will be
@@ -595,12 +625,38 @@ class ExternalCommand(PropertyManager):
         """
         return self.stderr_stream.load()
 
+    @mutable_property
+    def stderr_file(self):
+        """
+        Capture the standard error stream to the given file handle.
+
+        When this property is set to a writable file object the standard error
+        stream of the external command is redirected to the given file. The
+        default value of this property is :data:`None`.
+
+        This can be useful to (semi) permanently store command output or to run
+        commands whose output is hidden but can be followed using `tail -f`_ if
+        the need arises. By setting :attr:`stdout_file` and :attr:`stderr_file`
+        to the same file object the output from both streams can be merged and
+        redirected to the same file. This accomplishes roughly the same thing
+        as setting :attr:`merge_streams` but leaves the caller in control of
+        the file.
+
+        If this property isn't set but :attr:`capture` is :data:`True` the
+        external command's output is captured to a temporary file that's
+        automatically cleaned up after the external command is finished and its
+        output has been cached (read into memory).
+
+        .. _tail -f: https://en.wikipedia.org/wiki/Tail_(Unix)#File_monitoring
+        """
+
     @property
     def stdout(self):
         """
-        The output of the external command on its standard output stream, a
-        :class:`python2:str` object (in Python 2) or a :class:`python3:bytes`
-        object (in Python 3).
+        The output of the external command on its standard output stream.
+
+        This is a :class:`python2:str` object (in Python 2) or a
+        :class:`python3:bytes` object (in Python 3).
 
         This is only available when :attr:`capture` is :data:`True`. If
         :attr:`capture` is not :data:`True` then :attr:`stdout` will be
@@ -608,11 +664,36 @@ class ExternalCommand(PropertyManager):
         """
         return self.stdout_stream.load()
 
+    @mutable_property
+    def stdout_file(self):
+        """
+        Capture the standard output stream to the given file handle.
+
+        When this property is set to a writable file object the standard output
+        stream of the external command is redirected to the given file. The
+        default value of this property is :data:`None`.
+
+        This can be useful to (semi) permanently store command output or to run
+        commands whose output is hidden but can be followed using `tail -f`_ if
+        the need arises. By setting :attr:`stdout_file` and :attr:`stderr_file`
+        to the same file object the output from both streams can be merged and
+        redirected to the same file. This accomplishes roughly the same thing
+        as setting :attr:`merge_streams` but leaves the caller in control of
+        the file.
+
+        If this property isn't set but :attr:`capture` is :data:`True` the
+        external command's output is captured to a temporary file that's
+        automatically cleaned up after the external command is finished and its
+        output has been cached (read into memory).
+        """
+
     @property
     def succeeded(self):
         """
+        Whether the external command succeeded.
+
         :data:`True` if :attr:`returncode` is zero, :data:`False` if
-        :attr:`returncode` is set but not zero, :data:`None` when the external
+        :attr:`returncode` is a nonzero number, :data:`None` when the external
         command hasn't been started or is still running.
         """
         return self.returncode == 0 if self.is_finished else None
@@ -620,6 +701,8 @@ class ExternalCommand(PropertyManager):
     @mutable_property
     def sudo(self):
         """
+        Whether ``sudo`` should be used to gain superuser privileges.
+
         If this option is :data:`True` (not the default) and the current
         process doesn't have `superuser privileges`_ the external command is
         run with ``sudo`` to ensure that the external command runs with
@@ -630,6 +713,8 @@ class ExternalCommand(PropertyManager):
     @property
     def was_started(self):
         """
+        Whether the external command has already been started.
+
         :data:`True` once :func:`start()` has been called to start executing
         the external command, :data:`False` when :func:`start()` hasn't been
         called yet.
@@ -673,7 +758,10 @@ class ExternalCommand(PropertyManager):
             kw['stdout'] = self.null_device
             kw['stderr'] = self.null_device
         # Prepare to capture the standard output stream.
-        if self.capture:
+        if self.stdout_file:
+            self.stdout_stream.redirect(self.stdout_file)
+            kw['stdout'] = self.stdout_stream.fd
+        elif self.capture:
             if self.async:
                 self.stdout_stream.prepare()
                 kw['stdout'] = self.stdout_stream.fd
@@ -682,6 +770,9 @@ class ExternalCommand(PropertyManager):
         # Make it possible to merge stderr into stdout.
         if self.merge_streams:
             kw['stderr'] = subprocess.STDOUT
+        elif self.stderr_file:
+            self.stderr_stream.redirect(self.stderr_file)
+            kw['stderr'] = self.stderr_stream.fd
         elif self.capture_stderr:
             # Prepare to capture the standard error stream.
             if self.async:
@@ -735,9 +826,10 @@ class ExternalCommand(PropertyManager):
 
     def terminate(self):
         """
-        Terminate a running process using :func:`subprocess.Popen.terminate()`.
+        Terminate a running process.
 
-        Calls :func:`wait()` after terminating the process so that the external
+        Uses the :func:`subprocess.Popen.terminate()` function. Calls
+        :func:`wait()` after terminating the process so that the external
         command's output is loaded and temporary resources are cleaned up. The
         value of :attr:`check` is overridden to :data:`False` during the call
         to :func:`terminate()` (if you're terminating an external command you
@@ -751,7 +843,9 @@ class ExternalCommand(PropertyManager):
 
     def load_output(self):
         """
-        Reads the contents of the temporary file created by :func:`start()`
+        Load output captured from the standard output/error streams.
+
+        Reads the contents of the temporary file(s) created by :func:`start()`
         (when :attr:`async` and :attr:`capture` are both set) into memory so
         that the output doesn't get lost when the temporary file is cleaned up
         by :func:`cleanup()`.
@@ -776,8 +870,10 @@ class ExternalCommand(PropertyManager):
 
     def check_errors(self, check=None):
         """
-        Raise :attr:`error_type` when :attr:`check` is set and the external
-        command failed.
+        Raise an exception if the external command failed.
+
+        This raises :attr:`error_type` when :attr:`check` is set and the
+        external command failed.
 
         :param check: Override the value of :attr:`check` for the duration of
                       this call. Defaults to :data:`None` which means
@@ -799,8 +895,12 @@ class ExternalCommand(PropertyManager):
 
     def __exit__(self, exc_type=None, exc_value=None, traceback=None):
         """
-        Terminate the external command if it is still running, cleanup after
-        the external command and check for errors.
+        Automatically terminate and clean up after the external command.
+
+        Terminates the external command if it is still running (using
+        :func:`terminate()`), cleans up (using :func:`cleanup()`) and checks
+        for errors (using :func:`check_errors()`, only if an exception is not
+        already being handled).
         """
         if self.was_started:
             if self.is_running:
@@ -826,8 +926,28 @@ class CachedStream(object):
         self.cached_output = None
         self.fd = None
         self.filename = None
+        self.is_temporary_file = False
         self.kind = kind
 
+    def redirect(self, obj):
+        """
+        Capture the stream in a file provided by the caller.
+
+        :param obj: A file-like object that has an associated file descriptor.
+        """
+        # Try to get the file descriptor.
+        try:
+            self.fd = obj.fileno()
+        except Exception:
+            msg = "Can't capture %s stream to file object without file descriptor!"
+            raise ValueError(msg % self.kind)
+        # Try to get the filename.
+        self.filename = getattr(obj, 'name', None)
+        if not self.filename:
+            msg = "Can't capture %s stream to file object without filename! ('name' attribute)"
+            raise ValueError(msg % self.kind)
+        logger.debug("Connected %s stream to file %s ..", self.kind, self.filename)
+
     def prepare(self, contents=None):
         """
         Initialize the temporary file (and write the given string to it).
@@ -836,8 +956,9 @@ class CachedStream(object):
                          written to the temporary file.
         """
         if not (self.fd and self.filename):
+            self.is_temporary_file = True
             self.fd, self.filename = tempfile.mkstemp(prefix='executor-', suffix='-%s.txt' % self.kind)
-        logger.debug("Connecting %s stream to temporary file %s ..", self.kind, self.filename)
+            logger.debug("Connected %s stream to temporary file %s ..", self.kind, self.filename)
         if contents is not None:
             with open(self.filename, 'wb') as handle:
                 handle.write(contents)
@@ -865,9 +986,10 @@ class CachedStream(object):
 
     def cleanup(self):
         """Cleanup the temporary file."""
-        if self.filename:
+        if self.filename and self.is_temporary_file:
             if os.path.isfile(self.filename):
                 os.unlink(self.filename)
+            self.is_temporary_file = False
             self.filename = None
 
 
@@ -928,9 +1050,12 @@ def which(program):
 class ExternalCommandFailed(Exception):
 
     """
-    Raised by :func:`execute()`, :func:`~ExternalCommand.start()` and
-    :func:`~ExternalCommand.wait()` when an external command exits with a
-    nonzero status code. Exposes the following attributes:
+    Raised when an external command exits with a nonzero status code.
+
+    This exception is raised by :func:`execute()`,
+    :func:`~ExternalCommand.start()` and :func:`~ExternalCommand.wait()` when
+    an external command exits with a nonzero status code. Exposes the following
+    attributes:
 
     .. attribute:: command
 
@@ -938,6 +1063,11 @@ class ExternalCommandFailed(Exception):
     """
 
     def __init__(self, command):
+        """
+        Initialize an :class:`ExternalCommandFailed` object.
+
+        :param command: The :class:`ExternalCommand` object.
+        """
         self.command = command
         super(ExternalCommandFailed, self).__init__(command.error_message)
 
