@@ -1,9 +1,11 @@
 # Automated tests for the `executor' module.
 #
 # Author: Peter Odding <peter@peterodding.com>
-# Last Change: May 27, 2015
+# Last Change: October 2, 2015
 # URL: https://executor.readthedocs.org
 
+"""Automated tests for the `executor` package."""
+
 # Standard library modules.
 import logging
 import os
@@ -45,9 +47,13 @@ from executor.ssh.client import (
 )
 from executor.ssh.server import SSHServer
 
+
 class ExecutorTestCase(unittest.TestCase):
 
+    """Container for `executor` tests (methods)."""
+
     def setUp(self):
+        """Set up (colored) logging to the terminal."""
         try:
             # Optional external dependency.
             import coloredlogs
@@ -57,13 +63,16 @@ class ExecutorTestCase(unittest.TestCase):
             logging.basicConfig(level=logging.DEBUG)
 
     def test_argument_validation(self):
+        """Make sure the external command constructor requires a command argument."""
         self.assertRaises(TypeError, ExternalCommand)
 
     def test_program_searching(self):
+        """Make sure which() works as expected."""
         self.assertTrue(which('python'))
         self.assertFalse(which('a-program-name-that-no-one-would-ever-use'))
 
     def test_status_code_checking(self):
+        """Make sure that status code handling is sane."""
         self.assertTrue(execute('true'))
         self.assertFalse(execute('false', check=False))
         self.assertRaises(ExternalCommandFailed, execute, 'false')
@@ -79,9 +88,11 @@ class ExecutorTestCase(unittest.TestCase):
             self.assertEqual(e.returncode, 42)
 
     def test_stdin(self):
+        """Make sure standard input can be provided to external commands."""
         self.assertEqual(execute('tr', 'a-z', 'A-Z', input='test', capture=True), 'TEST')
 
     def test_stdout(self):
+        """Make sure standard output of external commands can be captured."""
         self.assertEqual(execute('echo', 'this is a test', capture=True), 'this is a test')
         self.assertEqual(execute('echo', '-e', r'line 1\nline 2', capture=True), 'line 1\nline 2\n')
         # I don't know how to test for the effect of silent=True in a practical
@@ -91,6 +102,7 @@ class ExecutorTestCase(unittest.TestCase):
         self.assertTrue(execute('echo', 'this is a test', silent=True))
 
     def test_stderr(self):
+        """Make sure standard error of external commands can be captured."""
         stdout_value = 'this goes to standard output'
         stderr_value = 'and this goes to the standard error stream'
         shell_command = 'echo %s; echo %s >&2' % (stdout_value, stderr_value)
@@ -100,6 +112,7 @@ class ExecutorTestCase(unittest.TestCase):
         assert stderr_value in cmd.decoded_stderr
 
     def test_merged_streams(self):
+        """Make sure standard output/error of external commands can be captured together."""
         stdout_value = 'this goes to standard output'
         stderr_value = 'and this goes to the standard error stream'
         shell_command = 'echo %s; echo %s >&2' % (stdout_value, stderr_value)
@@ -110,7 +123,82 @@ class ExecutorTestCase(unittest.TestCase):
         assert stdout_value not in (cmd.decoded_stderr or '')
         assert stderr_value not in (cmd.decoded_stderr or '')
 
+    def test_stdout_to_file(self):
+        """Make sure the standard output stream of external commands can be redirected and appended to a file."""
+        fd, filename = tempfile.mkstemp(prefix='executor-', suffix='-stdout.txt')
+        with open(filename, 'w') as handle:
+            handle.write('existing contents\n')
+        with open(filename, 'a') as handle:
+            execute('echo appended output', stdout_file=handle)
+        # Make sure the file was _not_ removed.
+        assert os.path.isfile(filename)
+        # Make sure the output was appended.
+        with open(filename) as handle:
+            lines = [line.strip() for line in handle]
+        assert lines == ['existing contents', 'appended output']
+
+    def test_stderr_to_file(self):
+        """Make sure the standard error stream of external commands can be redirected and appended to a file."""
+        fd, filename = tempfile.mkstemp(prefix='executor-', suffix='-stderr.txt')
+        with open(filename, 'w') as handle:
+            handle.write('existing contents\n')
+        with open(filename, 'a') as handle:
+            execute('echo appended output 1>&2', stderr_file=handle)
+        # Make sure the file was _not_ removed.
+        assert os.path.isfile(filename)
+        # Make sure the output was appended.
+        with open(filename) as handle:
+            lines = [line.strip() for line in handle]
+        assert lines == ['existing contents', 'appended output']
+
+    def test_merged_streams_to_file(self):
+        """Make sure the standard streams of external commands can be merged, redirected and appended to a file."""
+        fd, filename = tempfile.mkstemp(prefix='executor-', suffix='-merged.txt')
+        with open(filename, 'w') as handle:
+            handle.write('existing contents\n')
+        with open(filename, 'a') as handle:
+            execute('echo standard output; echo standard error 1>&2', stdout_file=handle, stderr_file=handle)
+        # Make sure the file was _not_ removed.
+        assert os.path.isfile(filename)
+        # Make sure the output was appended.
+        with open(filename) as handle:
+            lines = [line.strip() for line in handle]
+        assert lines == ['existing contents', 'standard output', 'standard error']
+
+    def test_asynchronous_stream_to_file(self):
+        """Make sure the standard streams can be redirected to a file and asynchronously stream output to that file."""
+        fd, filename = tempfile.mkstemp(prefix='executor-', suffix='-streaming.txt')
+        with open(filename, 'w') as handle:
+            cmd = ExternalCommand('for ((i=0; i<25; i++)); do command echo $i; sleep 0.1; done',
+                                  async=True, stdout_file=handle)
+            cmd.start()
+
+        def expect_some_output():
+            """Expect some but not all output to be readable at some point."""
+            with open(filename) as handle:
+                lines = list(handle)
+                assert len(lines) > 0
+                assert len(lines) < 25
+
+        def expect_most_output():
+            """Expect most but not all output to be readable at some point."""
+            with open(filename) as handle:
+                lines = list(handle)
+                assert len(lines) > 15
+                assert len(lines) < 25
+
+        def expect_all_output():
+            """Expect all output to be readable at some point."""
+            with open(filename) as handle:
+                lines = list(handle)
+                assert len(lines) == 25
+
+        retry(expect_some_output, 10)
+        retry(expect_most_output, 20)
+        retry(expect_all_output, 30)
+
     def test_working_directory(self):
+        """Make sure the working directory of external commands can be set."""
         directory = tempfile.mkdtemp()
         try:
             self.assertEqual(execute('echo $PWD', capture=True, directory=directory), directory)
@@ -118,6 +206,7 @@ class ExecutorTestCase(unittest.TestCase):
             os.rmdir(directory)
 
     def test_fakeroot_option(self):
+        """Make sure ``fakeroot`` can be used."""
         filename = os.path.join(tempfile.gettempdir(), 'executor-%s-fakeroot-test' % os.getpid())
         self.assertTrue(execute('touch', filename, fakeroot=True))
         try:
@@ -130,6 +219,7 @@ class ExecutorTestCase(unittest.TestCase):
             os.unlink(filename)
 
     def test_sudo_option(self):
+        """Make sure ``fakeroot`` can be used."""
         filename = os.path.join(tempfile.gettempdir(), 'executor-%s-sudo-test' % os.getpid())
         self.assertTrue(execute('touch', filename))
         try:
@@ -142,13 +232,18 @@ class ExecutorTestCase(unittest.TestCase):
             self.assertTrue(execute('rm', filename, sudo=True))
 
     def test_environment_variable_handling(self):
+        """Make sure environment variables can be overridden."""
         # Check that environment variables of the current process are passed on to subprocesses.
         self.assertEqual(execute('echo $PATH', capture=True), os.environ['PATH'])
         # Test that environment variable overrides can be given to external commands.
         override_value = str(random.random())
-        self.assertEqual(execute('echo $override', capture=True, environment=dict(override=override_value)), override_value)
+        self.assertEqual(execute('echo $override',
+                                 capture=True,
+                                 environment=dict(override=override_value)),
+                         override_value)
 
     def test_simple_async_cmd(self):
+        """Make sure commands can be executed asynchronously."""
         cmd = ExternalCommand('sleep 4', async=True)
         # Make sure we're starting from a sane state.
         assert not cmd.was_started
@@ -156,14 +251,19 @@ class ExecutorTestCase(unittest.TestCase):
         assert not cmd.is_finished
         # Start the external command.
         cmd.start()
-        # Make sure the external command switches to the running state within a
-        # reasonable time (this is sensitive to timing issues on slow or
-        # overloaded systems, the retry logic is there to make the test pass as
-        # quickly as possible while still allowing for some delay).
+
         def assert_running():
+            """
+            Make sure command switches to running state within a reasonable time.
+
+            This is sensitive to timing issues on slow or overloaded systems,
+            the retry logic is there to make the test pass as quickly as
+            possible while still allowing for some delay.
+            """
             assert cmd.was_started
             assert cmd.is_running
             assert not cmd.is_finished
+
         retry(assert_running, timeout=4)
         # Wait for the external command to finish.
         cmd.wait()
@@ -174,6 +274,7 @@ class ExecutorTestCase(unittest.TestCase):
         assert cmd.returncode == 0
 
     def test_async_with_input(self):
+        """Make sure asynchronous commands can be provided standard input."""
         random_file = os.path.join(tempfile.gettempdir(), 'executor-%s-async-input-test' % os.getpid())
         random_value = str(random.random())
         cmd = ExternalCommand('cat > %s' % quote(random_file), async=True, input=random_value)
@@ -189,6 +290,7 @@ class ExecutorTestCase(unittest.TestCase):
                 os.unlink(random_file)
 
     def test_async_with_output(self):
+        """Make sure asynchronous command output can be captured."""
         random_value = str(random.random())
         cmd = ExternalCommand('echo %s' % quote(random_value), async=True, capture=True)
         cmd.start()
@@ -196,24 +298,33 @@ class ExecutorTestCase(unittest.TestCase):
         assert cmd.output == random_value
 
     def test_repr(self):
-        cmd = ExternalCommand('echo 42', async=True, capture=True, directory='/', environment={'my-environment-variable': '42'})
+        """Make sure that repr() on external commands gives sane output."""
+        cmd = ExternalCommand('echo 42',
+                              async=True,
+                              capture=True,
+                              directory='/',
+                              environment={'my_environment_variable': '42'})
         assert repr(cmd).startswith('ExternalCommand(')
         assert repr(cmd).endswith(')')
         assert 'echo 42' in repr(cmd)
         assert 'async=True' in repr(cmd)
         assert ('directory=%r' % '/') in repr(cmd)
-        assert 'my-environment-variable' in repr(cmd)
+        assert 'my_environment_variable' in repr(cmd)
         assert 'was_started=False' in repr(cmd)
         assert 'is_running=False' in repr(cmd)
         assert 'is_finished=False' in repr(cmd)
         cmd.start()
+
         def assert_finished():
+            """Allow for some delay before the external command finishes."""
             assert 'was_started=True' in repr(cmd)
             assert 'is_running=False' in repr(cmd)
             assert 'is_finished=True' in repr(cmd)
+
         retry(assert_finished, 10)
 
     def test_command_pool(self):
+        """Make sure command pools actually run multiple commands in parallel."""
         num_commands = 10
         sleep_time = 4
         pool = CommandPool(5)
@@ -225,6 +336,7 @@ class ExecutorTestCase(unittest.TestCase):
         assert timer.elapsed_time < (num_commands * sleep_time)
 
     def test_ssh_command_lines(self):
+        """Make sure SSH client command lines are correctly generated."""
         # Construct a remote command using as much defaults as possible and
         # validate the resulting SSH client program command line.
         cmd = RemoteCommand('localhost', 'true', ssh_user='some-random-user')
@@ -273,6 +385,7 @@ class ExecutorTestCase(unittest.TestCase):
             tokenize_command_line(RemoteCommand('localhost', 'date', ssh_user='root', sudo=True))
 
     def test_ssh_unreachable(self):
+        """Make sure a specific exception is raised when ``ssh`` fails to connect."""
         # Make sure invalid SSH aliases raise the expected type of exception.
         self.assertRaises(
             RemoteConnectFailed,
@@ -280,18 +393,25 @@ class ExecutorTestCase(unittest.TestCase):
         )
 
     def test_remote_working_directory(self):
+        """Make sure remote working directories can be set."""
         with SSHServer(async=True) as server:
             some_random_directory = tempfile.mkdtemp()
-            cmd = RemoteCommand('127.0.0.1', 'pwd', capture=True, directory=some_random_directory, **server.client_options)
+            cmd = RemoteCommand('127.0.0.1',
+                                'pwd',
+                                capture=True,
+                                directory=some_random_directory,
+                                **server.client_options)
             cmd.start()
             assert cmd.output == some_random_directory
 
     def test_remote_error_handling(self):
+        """Make sure remote commands preserve exit codes."""
         with SSHServer(async=True) as server:
             cmd = RemoteCommand('127.0.0.1', 'exit 42', **server.client_options)
             self.assertRaises(RemoteCommandFailed, cmd.start)
 
     def test_foreach(self):
+        """Make sure remote command pools work."""
         with SSHServer(async=True) as server:
             ssh_aliases = ['127.0.0.%i' % i for i in (1, 2, 3, 4, 5, 6, 7, 8)]
             results = foreach(ssh_aliases, 'echo $SSH_CONNECTION',
@@ -301,13 +421,16 @@ class ExecutorTestCase(unittest.TestCase):
             assert len(ssh_aliases) == len(set(cmd.output for cmd in results))
 
     def test_local_context(self):
+        """Test a local command context."""
         self.check_context(LocalContext())
 
     def test_remote_context(self):
+        """Test a remote command context."""
         with SSHServer(async=True) as server:
             self.check_context(RemoteContext('127.0.0.1', **server.client_options))
 
     def check_context(self, context):
+        """Test a command execution context (whether local or remote)."""
         # Make sure __str__() does something useful.
         assert 'system' in str(context)
         # Test context.execute() and cleanup().
@@ -425,14 +548,17 @@ class CustomPropertyTestCase(unittest.TestCase):
 
 
 def tokenize_command_line(cmd):
+    """Tokenize a command line string into a list of strings."""
     return sum(map(shlex.split, cmd.command_line), [])
 
 
 def retry(func, timeout):
+    """Retry a function until it no longer raises assertion errors or time runs out before then."""
     time_started = time.time()
-    while (time.time() - time_started) < timeout:
+    while True:
+        timeout_expired = (time.time() - time_started) >= timeout
         try:
             return func()
         except AssertionError:
-            pass
-    assert False, "Timeout expired but function never passed all assertions!"
+            if timeout_expired:
+                raise
